// A structure for our custom vertex type
struct ZUIVertex
{
    FLOAT x, y, z, rhw; // The transformed position for the vertex
    DWORD color;        // The vertex color
    float u,v;
};

// Our custom FVF, which describes our custom vertex structure
#define FVF_ZUIVERTEX   (D3DFVF_XYZRHW | D3DFVF_DIFFUSE | D3DFVF_TEX1 | D3DFVF_TEXCOORDSIZE2(0))

// A structure for our custom vertex type
struct ZUILineVertex
{
    FLOAT x, y, z, rhw; // The transformed position for the vertex
    DWORD color;        // The vertex color
};

// Our custom FVF, which describes our custom vertex structure
#define FVF_ZUILINEVERTEX   (D3DFVF_XYZRHW | D3DFVF_DIFFUSE )


enum ZUITYPES
{
    ZUI_PICTURE = 0,
    ZUI_EDITBOX = 1,
    ZUI_CHECKBOX = 2,
    ZUI_DROPDOWN = 3,
    ZUI_BUTTON = 4,
    ZUI_SLIDER = 5
};

extern LPDIRECT3DDEVICE8 g_pDevice;

class CZUIItem
{
public:
    float width,height;
    float x,y,z;
    ZUIVertex *m_pVerts;

    D3DCOLOR color;
    unsigned int m_texID;
    unsigned int m_ID;
    unsigned int m_type;
    std::string m_name;
    unsigned int m_iTranslucency;
    bool m_bCallbackEvent;

    CZUIItem()
    {
        m_pVerts = NULL;
        m_iTranslucency = 255;
        m_bCallbackEvent = false;
    }

    virtual ~CZUIItem()
    {
        if(m_pVerts)
            delete m_pVerts;
        m_pVerts = NULL;
    }

    void CreateItem(float _x,float _y,float _z,
                    float _width,float _height,
                    D3DCOLOR _color = 0xFFFFFFFF)
    {
        x = _x; y = _y; z = _z;
        width = _width; height = _height;
        color = _color;

        if(m_pVerts)
            delete m_pVerts;

        m_pVerts = new ZUIVertex[4];

        Initialize();
    }

    void Initialize()
    {
        ZUIVertex *pVerts = m_pVerts;

        pVerts->x = x;
        pVerts->y = y;
        pVerts->z = z;
        pVerts->rhw = z;
        pVerts->u = 0;
        pVerts->v = 0;
        pVerts->color = D3DCOLOR_ARGB(m_iTranslucency,255,255,255);
        pVerts++;

        pVerts->x = x+width;
        pVerts->y = y;
        pVerts->z = z;
        pVerts->rhw = z;
        pVerts->u = 1;
        pVerts->v = 0;
        pVerts->color = D3DCOLOR_ARGB(m_iTranslucency,255,255,255);
        pVerts++;

        pVerts->x = x;
        pVerts->y = y+height;
        pVerts->z = z;
        pVerts->rhw = z;
        pVerts->u = 0;
        pVerts->v = 1;
        pVerts->color = D3DCOLOR_ARGB(m_iTranslucency,255,255,255);
        pVerts++;

        pVerts->x = x+width;
        pVerts->y = y+height;
        pVerts->z = z;
        pVerts->rhw = z;
        pVerts->u = 1;
        pVerts->v = 1;
        pVerts->color = D3DCOLOR_ARGB(m_iTranslucency,255,255,255);
    }

    bool operator < (const CZUIItem &a)
    {
        return m_texID < a.m_texID;
    }

    inline virtual bool IsA(ZUITYPES _type)
    {
        if(_type == ZUI_PICTURE)
            return true;
        return false;
    }

    virtual void MouseMove(int x,int y){}
    virtual void MouseDown(int button,int x,int y){}
    virtual void MouseUp(int button,int x,int y){}
    virtual void MouseOff(){}
};

class CZUIEditBox : public CZUIItem
{
public:
    unsigned int *m_texIDs;
    std::string m_text;
    unsigned int m_fontID;
    int m_sel;

    CZUIEditBox()
    {
        m_texIDs = new unsigned int[3];
        m_text = "test";
        m_fontID = 0;
        m_sel = -1;
    }

    ~CZUIEditBox()
    {
        delete[] m_texIDs;
    }

    inline virtual bool IsA(ZUITYPES _type)
    {
        if(_type == ZUI_EDITBOX)
            return true;
        return false;
    }
    
    void SetMaps(int mapBack,int mapFront,int mapHilight)
    {
        m_texIDs[0] = mapBack;
        m_texIDs[1] = mapFront;
        m_texIDs[2] = mapHilight;
    }

    void SetSel(int x)
    {
        m_sel = x;
    }
};

class CZUICheckBox : public CZUIItem
{
public:
    unsigned int *m_texIDs;
    bool checked;

    CZUICheckBox()
    {
        m_texIDs = new unsigned int[6];
        checked = false;
    }
    ~CZUICheckBox()
    {
        delete[] m_texIDs;
    }
    inline virtual bool IsA(ZUITYPES _type)
    {
        if(_type == ZUI_CHECKBOX)
            return true;
        return false;
    }

    void SetMaps(int mapNormal,int mapChecked,
                  int mapHover,int mapHoverChecked,
                  int mapHoverDown,int mapHoverDownChecked)
    {
        m_texIDs[0] = mapNormal;
        m_texIDs[1] = mapChecked;
        m_texIDs[2] = mapHover;
        m_texIDs[3] = mapHoverChecked;
        m_texIDs[4] = mapHoverDown;
        m_texIDs[5] = mapHoverDownChecked;
        m_texID = m_texIDs[0];
    }

    virtual void MouseMove(int,int)
    {
        if(checked)
        {
            m_texID = m_texIDs[3];
        }
        else
        {
            m_texID = m_texIDs[2];
        }

    }
    virtual void MouseDown(int,int,int)
    {
        if(checked)
        {
            m_texID = m_texIDs[5];
        }
        else
        {
            m_texID = m_texIDs[4];
        }
    }
    virtual void MouseUp(int,int,int)
    {
        if(checked)
        {
            checked = false;
            m_texID = m_texIDs[2];
        }
        else
        {
            checked = true;
            m_texID = m_texIDs[3];
        }
    }
    virtual void MouseOff()
    {
        if(checked)
        {
            m_texID = m_texIDs[1];
        }
        else
        {
            m_texID = m_texIDs[0];
        }
    }
};

class CZUIButton : public CZUIItem
{
public:
    unsigned int *m_texIDs;
    std::string m_text;
    unsigned int m_fontID;

    CZUIButton()
    {
        m_texIDs = new unsigned int[2];
    }
    ~CZUIButton()
    {
        delete[] m_texIDs;
    }

    inline virtual bool IsA(ZUITYPES _type)
    {
        if(_type == ZUI_BUTTON)
            return true;
        return false;
    }

    virtual void MouseDown(int ,int,int)
    {
        m_texID = m_texIDs[1];
    }
    virtual void MouseOff()
    {
        m_texID = m_texIDs[0];
    }
    virtual void MouseUp(int,int,int)
    {
        m_texID = m_texIDs[0];
    }
};

class CZUISlider : public CZUIItem
{
public:
    unsigned int *m_texIDs;
    int sliderPos;
    int sliderMin;
    int sliderMax;

    float sliderWidth;
    float sliderHeight;

    CZUISlider()
    {
        m_texIDs = new unsigned int[2];
    }
    ~CZUISlider()
    {
        delete[] m_texIDs;
    }

    inline virtual bool IsA(ZUITYPES _type)
    {
        if(_type == ZUI_SLIDER)
            return true;
        return false;
    }

    virtual void MouseDown(int,int _x,int)
    {
        float xoffset = _x - x;

        xoffset = (xoffset / width) * sliderMax;
        
        if(xoffset > sliderMax)
            xoffset = sliderMax;
        
        sliderPos = xoffset;
    }
};