struct greaterCZUIItemTex
{
  bool operator()(const CZUIItem * lhs, const CZUIItem * rhs) const
  {
     return lhs->m_pVerts[0].rhw > rhs->m_pVerts[0].rhw;
  }
};

class CZUIManager
{
public:

    bool m_bMouseDown;
    int  m_iLastCaptureItem;
    int  m_iCaptureItem;
    int  m_iMouseOverItemID;
    

    std::vector<CZUIItem *> m_ItemList;
    std::vector<CZUITexture *> m_TextureList;
    std::vector<CZUIFont *>m_FontList;
    
    unsigned int m_nextTexID;
    unsigned int m_nextItemID;
    unsigned int m_nextFontID;
    
    unsigned int m_totalTextures;
    unsigned int m_totalItems;
    unsigned int m_totalFonts;
    int m_opacity;

    int m_virtualWidth;
    int m_virtualHeight;
    int m_xoffset;
    int m_yoffset;

    void (*eventCallback)(CZUIItem*);

    CZUIFont *AllocFont()
    {
        CZUIFont *pNewFont = new CZUIFont;
        pNewFont->m_ID = m_nextFontID++;
        m_FontList.push_back(pNewFont);
        m_totalFonts++;
        return pNewFont;
    }

    CZUIItem *AllocItem(int itemType)
    {
        CZUIItem *pNewItem;
        switch(itemType)
        {
            case ZUI_CHECKBOX:
                {
                    pNewItem = new CZUICheckBox;
                }
                break;
            case ZUI_EDITBOX:
                {
                    pNewItem = new CZUIEditBox;
                }
                break;
            case ZUI_BUTTON:
                {
                    pNewItem = new CZUIButton;
                }
                break;
            case ZUI_SLIDER:
                {
                    pNewItem = new CZUISlider;
                }
                break;
            default:
                {
                    pNewItem = new CZUIItem;
                }
                break;
        }
        
        m_ItemList.push_back(pNewItem);
        m_totalItems++;
        return pNewItem;
    }

    CZUITexture *AllocTexture()
    {
        CZUITexture *pNewTex = new CZUITexture;
        pNewTex->m_ID = m_nextTexID++;
        m_TextureList.push_back(pNewTex);
        m_totalTextures++;
        return pNewTex;
    }
public:

    CZUIManager()
    {
        m_nextTexID = 0;
        m_nextItemID = 0;
        m_nextFontID = 0;

        m_totalItems=0;
        m_totalTextures=0;
        m_totalFonts = 0;
        m_opacity = 120;

        m_iCaptureItem = -1;
        m_bMouseDown = false;
        m_iMouseOverItemID = -1;

        m_virtualWidth = 800;
        m_virtualHeight = 600;

        m_xoffset = 0;
        m_yoffset = 0;

        eventCallback = NULL;
    }

    ~CZUIManager()
    {
        RemoveAll();
    }

    void RemoveAll()
    {
        m_nextTexID = 0;
        m_nextItemID = 0;
        m_nextFontID = 0;

        m_totalItems = 0;
        m_totalTextures = 0;
        m_totalFonts = 0;
        
        //remove items
        std::vector<CZUIItem *>::iterator iterItems = m_ItemList.begin();
        for(;iterItems != m_ItemList.end(); iterItems++)
            delete (*iterItems);
        m_ItemList.clear();

        //remove textures
        std::vector<CZUITexture *>::iterator iterTex = m_TextureList.begin();
        for(;iterTex != m_TextureList.end(); iterTex++)
            delete (*iterTex);
        m_TextureList.clear();

        //remove fonts
        std::vector<CZUIFont *>::iterator iterFont = m_FontList.begin();
        for(;iterFont != m_FontList.end(); iterFont++)
            delete (*iterFont);
        m_FontList.clear();
    }
    
    int CreateTexture(const char *fileName,bool translucent = false)
    {
        std::vector<CZUITexture *>::iterator iterTex = m_TextureList.begin();
        for(;iterTex != m_TextureList.end(); iterTex++)
        {
            CZUITexture *pTex = (*iterTex);
            if(pTex->m_fileName == fileName)
            {
                return pTex->m_ID;
            }
        }
        CZUITexture *pTex = AllocTexture();
        pTex->m_fileName = fileName;
        pTex->translucent = translucent;
        pTex->LoadTexture(fileName);
        return pTex->m_ID;
    }

    // initialize items after removing a texture
    void RemoveTexture(int texID)
    {
        std::vector<CZUITexture *>::iterator iterTex = m_TextureList.begin();
        for(;iterTex != m_TextureList.end(); iterTex++)
        {
            CZUITexture *pTex = (*iterTex);
            if(pTex->m_ID == texID)
            {
                //remove items with this texture
                //remove items
                std::vector<CZUIItem *>::iterator iterItems = m_ItemList.begin();
                for(;iterItems != m_ItemList.end();)
                {
                    CZUIItem *pItem = (*iterItems);
                    if(pItem->m_texID == texID)
                    {
                        delete (*iterItems);
                        m_ItemList.erase(iterItems);
                    }
                    else
                    {
                        iterItems++;
                    }
                }

                delete (*iterTex);
                m_TextureList.erase(iterTex);
                return;
            }
        }
    }

    CZUIFont *GetFont(unsigned int fontID)
    {
        std::vector<CZUIFont*>::iterator iter = m_FontList.begin();
        for(;iter != m_FontList.end(); iter++)
        {
            CZUIFont *pFont = (*iter);
            if(pFont->m_ID == fontID)
            {
                return pFont;
            }
        }
        return NULL;
    }

    CZUIFont *GetFont(std::string fontName)
    {
        std::vector<CZUIFont*>::iterator iter = m_FontList.begin();
        for(;iter != m_FontList.end(); iter++)
        {
            CZUIFont *pFont = (*iter);
            if(pFont->m_fontName == fontName)
            {
                return pFont;
            }
        }
        return NULL;
    }

    void RemoveFont(unsigned int fontID)
    {
        if(GetFont(fontID))
        {
            std::vector<CZUIFont*>::iterator iter = m_FontList.begin();
            for(;iter != m_FontList.end(); iter++)
            {
                CZUIFont *pFont = (*iter);
                if(pFont->m_ID == fontID)
                {
                    delete pFont;
                    m_FontList.erase(iter);
                    m_totalFonts--;
                    return;
                }
            }
        }
    }

    int CreateFont(long rgb,const char *fontName,int fontSize,int fontStyle)
    {
        CZUIFont *pFont = AllocFont();
        pFont->m_ID = m_nextFontID++;
        pFont->SetFont(fontName,fontSize,rgb,fontStyle);
        return pFont->m_ID;
    }

    //call initialize items after creating a batch
    int CreateItem(float x,float y,float z,float width,float height,int texID,unsigned int itemType,int *texIDs = NULL)
    {
        CZUIItem *pItem = AllocItem(itemType);
        pItem->m_ID = m_nextItemID++;
        pItem->CreateItem(x,y,z,width,height);
        if(texIDs == NULL)
            pItem->m_texID = texID;
        else
            pItem->m_texID = texIDs[0];

        pItem->m_type = itemType;

        if(pItem->IsA(ZUI_CHECKBOX))
        {
             CZUICheckBox *pCheckbox = static_cast<CZUICheckBox*>(pItem);

             for(int x=0;x<6;x++)
             {
                pCheckbox->m_texIDs[x] = texIDs[x];
             }
        }
        else if(pItem->IsA(ZUI_EDITBOX))
        {
            CZUIEditBox *pEditbox = static_cast<CZUIEditBox*>(pItem);
            for(int x=0;x<2;x++)
            {
                pEditbox->m_texIDs[x] = texIDs[x];
            }
        }
        else if(pItem->IsA(ZUI_BUTTON))
        {
            CZUIButton *pButton = static_cast<CZUIButton*>(pItem);
            for(int x=0;x<2;x++)
            {
                pButton->m_texIDs[x] = texIDs[x];
            }
        }
        else if(pItem->IsA(ZUI_SLIDER))
        {
            CZUISlider *pSlider = static_cast<CZUISlider*>(pItem);
            for(int x=0;x<2;x++)
            {
                pSlider->m_texIDs[x] = texIDs[x];
            }
        }
        return pItem->m_ID;
    }

    //sort by texture -- internal
    void SortItems()
    {
        std::sort(m_ItemList.begin(),m_ItemList.end(),greaterCZUIItemTex());
    }


    void Render()
    {  
        g_pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE); 

        std::vector<CZUIItem *>::iterator iter = m_ItemList.begin();
        for(;iter != m_ItemList.end(); iter++)
        {
            CZUIItem *pItem = (*iter);

            g_pDevice->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
            g_pDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

            g_pDevice->SetVertexShader(FVF_ZUIVERTEX);

            
            if(pItem->IsA(ZUI_EDITBOX))
            {
                CZUIEditBox *pEdit = static_cast<CZUIEditBox*>(pItem);

                for(int x=0;x<2;x++)
                {
                    CZUITexture *pTex = GetTexture(pEdit->m_texIDs[x]);
                    pTex->m_opacity = m_opacity;
                    pTex->ActivateTexture();
                    g_pDevice->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
                    g_pDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
                    g_pDevice->SetVertexShader(FVF_ZUIVERTEX);
                    g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,(void*)pItem->m_pVerts,sizeof(ZUIVertex));
                    if(x == 0)
                    {
                        D3DVIEWPORT8 viewportNew;
                        D3DVIEWPORT8 viewportOld;

                        g_pDevice->GetViewport(&viewportOld);
                        memcpy(&viewportNew,&viewportOld,sizeof(D3DVIEWPORT8));
                        
                        viewportNew.Width = pItem->width;
                        viewportNew.Height = pItem->height;
                        viewportNew.X = pItem->x;
                        viewportNew.Y = pItem->y;

                        g_pDevice->SetViewport(&viewportNew);
                        
                        CZUIFont *pFont = GetFont(pEdit->m_fontID);
                        pFont->x = pEdit->x + 5;
                        pFont->y = pEdit->y;
                        pFont->m_text = pEdit->m_text;
            
                        
                        SIZE textSize;
                        pFont->m_pFont->GetTextExtent((char*)pEdit->m_text.c_str(),&textSize);
                        if(textSize.cx > pEdit->width)
                        {
                            pFont->x = pEdit->x +  ((pEdit->width - textSize.cx) - 10);
                        }
                        
                       // g_pDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
                        pFont->Render();
                        

                        if(pEdit->m_sel != -1)
                        {
                            g_pDevice->SetTexture(0,NULL);
                            g_pDevice->SetVertexShader(FVF_ZUILINEVERTEX);

                            g_pDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1);
                            g_pDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_DIFFUSE );
                            g_pDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
                            g_pDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_DISABLE );

                            ZUILineVertex editSelection[2];
                            editSelection[0].color = D3DCOLOR_XRGB(0,0,0);
                            editSelection[0].rhw = pItem->m_pVerts->rhw - .0001;
                            editSelection[0].x = pEdit->x + 5 + (pEdit->m_sel > pEdit->width?pEdit->width -10 : pEdit->m_sel) ;
                            editSelection[0].y = pEdit->y+3;
                            editSelection[0].z = pItem->m_pVerts->z - .0001;

                            editSelection[1].color = D3DCOLOR_XRGB(0,0,0);
                            editSelection[1].rhw = pItem->m_pVerts->rhw - .0001;
                            editSelection[1].x = pEdit->x + 5 + (pEdit->m_sel > pEdit->width?pEdit->width -10 : pEdit->m_sel) ;
                            editSelection[1].y = pEdit->y + textSize.cy-4;
                            editSelection[1].z = pItem->m_pVerts->z - .0001;

                            g_pDevice->DrawPrimitiveUP(D3DPT_LINELIST,1,&editSelection[0],sizeof(ZUILineVertex));
                        }

                        //g_pDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
                        g_pDevice->SetViewport(&viewportOld);
                    }

                }
            }
            else if(pItem->IsA(ZUI_BUTTON))
            {
                CZUIButton *pButton = static_cast<CZUIButton*>(pItem);

                CZUITexture *pTex = GetTexture(pItem->m_texID);
                pTex->m_opacity = m_opacity;
                pTex->ActivateTexture();
            
                g_pDevice->SetVertexShader(FVF_ZUIVERTEX);
                g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,(void*)pItem->m_pVerts,sizeof(ZUIVertex));

                D3DVIEWPORT8 viewportNew;
                D3DVIEWPORT8 viewportOld;

                g_pDevice->GetViewport(&viewportOld);
                memcpy(&viewportNew,&viewportOld,sizeof(D3DVIEWPORT8));
                
                viewportNew.Width = pItem->width;
                viewportNew.Height = pItem->height;
                viewportNew.X = pItem->x;
                viewportNew.Y = pItem->y;

                g_pDevice->SetViewport(&viewportNew);

                
                CZUIFont *pFont = GetFont(pButton->m_fontID);
                pFont->m_style &= D3DFONT_CENTERED;

                SIZE textSize;
                pFont->m_pFont->GetTextExtent((char*)pButton->m_text.c_str(),&textSize);

                pFont->x = (pItem->x + pItem->width/2) - textSize.cx / 2;
                pFont->y = (pItem->y + pItem->height/2) - textSize.cy / 2;
                pFont->m_text = pButton->m_text;

                pFont->Render();

                g_pDevice->SetViewport(&viewportOld);
            }

            else if(pItem->IsA(ZUI_SLIDER))
            {
                CZUISlider *pSlider = static_cast<CZUISlider*>(pItem);

                CZUITexture *pTex = GetTexture(pSlider->m_texIDs[0]);
                pTex->m_opacity = m_opacity;
                pTex->ActivateTexture();
                g_pDevice->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
                g_pDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
                g_pDevice->SetVertexShader(FVF_ZUIVERTEX);
                g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,(void*)pItem->m_pVerts,sizeof(ZUIVertex));


                int backWidth = pTex->m_width;

                pTex = GetTexture(pSlider->m_texIDs[1]);
                pTex->m_opacity = m_opacity;
                pTex->ActivateTexture();
                g_pDevice->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
                g_pDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
                g_pDevice->SetVertexShader(FVF_ZUIVERTEX);

                ZUIVertex pSliderVerts[4];
                memcpy(pSliderVerts,pSlider->m_pVerts,sizeof(ZUIVertex)*4);

                float sliderPerc = pSlider->sliderPos*(100.f/((pSlider->sliderMax + pTex->m_width/2)-pSlider->sliderMin));
                float offset = backWidth * (sliderPerc*.01);

                
                
                pSliderVerts[0].rhw -= .0001f;
                pSliderVerts[0].x += offset;
                pSliderVerts[2].x += offset;
                pSliderVerts[2].rhw -= .0001f;
                pSliderVerts[1].x = pSliderVerts[0].x + pTex->m_width;
                pSliderVerts[1].rhw -= .0001f;
                pSliderVerts[3].x = pSliderVerts[0].x + pTex->m_width;
                pSliderVerts[3].rhw -= .0001f;
            
                
                g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,(void*)pSliderVerts,sizeof(ZUIVertex));
            }
            else
            {
                CZUITexture *pTex = GetTexture(pItem->m_texID);
                pTex->m_opacity = m_opacity;
                pTex->ActivateTexture();
            
                g_pDevice->SetVertexShader(FVF_ZUIVERTEX);
                g_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,(void*)pItem->m_pVerts,sizeof(ZUIVertex));
            }
            g_pDevice->SetRenderState( D3DRS_ZWRITEENABLE, TRUE );
            g_pDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
        }
    }

    CZUITexture *GetTexture(int texID)
    {
        std::vector<CZUITexture *>::iterator iterTex = m_TextureList.begin();
        for(;iterTex != m_TextureList.end(); iterTex++)
        {
            CZUITexture *pTex = (*iterTex);
            if(pTex->m_ID == texID)
                return pTex;
        }
        return NULL;
    }

    CZUIItem *GetItem(int ItemID)
    {
        std::vector<CZUIItem *>::iterator iterItem = m_ItemList.begin();
        for(;iterItem != m_ItemList.end(); iterItem++)
        {
            CZUIItem *pItem = (*iterItem);
            if(pItem->m_ID == ItemID)
                return pItem;
        }
        return NULL;
    }

    void RemoveItem(int itemID)
    {
        std::vector<CZUIItem *>::iterator iterItem = m_ItemList.begin();
        for(;iterItem != m_ItemList.end(); iterItem++)
        {
            CZUIItem *pItem = (*iterItem);
            if(pItem->m_ID == itemID)
            {
                m_totalItems -= 1;
                delete pItem;
                m_ItemList.erase(iterItem);
                return;
            }
        }
    }

    void Save(const char *fileName)
    {
        FILE *fp = fopen(fileName,"wb");

        fwrite(&m_totalTextures,sizeof(unsigned int),1,fp);
        fwrite(&m_totalItems,sizeof(unsigned int),1,fp);
        fwrite(&m_totalFonts,sizeof(unsigned int),1,fp);
        fwrite(&m_virtualWidth,sizeof(int),1,fp);
        fwrite(&m_virtualHeight,sizeof(int),1,fp);

        std::vector<CZUITexture *>::iterator iter = m_TextureList.begin();
        for(;iter != m_TextureList.end(); iter++)
        {
            CZUITexture *pTex = (*iter);
            fwrite(&pTex->m_ID,sizeof(unsigned int),1,fp);
            int fileNameLen = pTex->m_fileName.length();
            fwrite(&fileNameLen,sizeof(int),1,fp);
            fwrite(pTex->m_fileName.c_str(),pTex->m_fileName.length(),1,fp);
            fwrite(&pTex->translucent,sizeof(bool),1,fp);
        }

        std::vector<CZUIFont*>::iterator iterFont = m_FontList.begin();
        for(;iterFont != m_FontList.end(); iterFont++)
        {
            CZUIFont *pFont= (*iterFont);
            fwrite(&pFont->m_ID,sizeof(unsigned int),1,fp);
            int fontNameLen = pFont->m_fontName.length();
            fwrite(&fontNameLen,sizeof(int),1,fp);
            fwrite(pFont->m_fontName.c_str(),pFont->m_fontName.length(),1,fp);
            fwrite(&pFont->m_color,sizeof(long),1,fp);
            fwrite(&pFont->m_size,sizeof(long),1,fp);
            fwrite(&pFont->m_style,sizeof(long),1,fp);
        }

        std::vector<CZUIItem *>::iterator iterItem = m_ItemList.begin();
        for(;iterItem != m_ItemList.end(); iterItem++)
        {
            CZUIItem *pItem = (*iterItem);
            fwrite(&pItem->m_ID,sizeof(unsigned int),1,fp);
            fwrite(&pItem->m_type,sizeof(unsigned int),1,fp);
            fwrite(&pItem->m_iTranslucency,sizeof(unsigned int),1,fp);
            fwrite(&pItem->m_bCallbackEvent,sizeof(bool),1,fp);

            if(pItem->IsA(ZUI_CHECKBOX))
            {
                CZUICheckBox *pCheckbox = static_cast<CZUICheckBox*>(pItem);
                for(int x=0;x<6;x++)
                {
                    fwrite(&pCheckbox->m_texIDs[x],sizeof(unsigned int),1,fp);
                }
            }
            else if(pItem->IsA(ZUI_EDITBOX))
            {
                CZUIEditBox *pEditbox = static_cast<CZUIEditBox*>(pItem);
                for(int x=0;x<2;x++)
                {
                    fwrite(&pEditbox->m_texIDs[x],sizeof(unsigned int),1,fp);
                }
                fwrite(&pEditbox->m_fontID,sizeof(unsigned int),1,fp);
            }
            else if(pItem->IsA(ZUI_BUTTON))
            {
                CZUIButton *pButton = static_cast<CZUIButton*>(pItem);
                for(int x=0;x<2;x++)
                {
                    fwrite(&pButton->m_texIDs[x],sizeof(unsigned int),1,fp);
                }
                fwrite(&pButton->m_fontID,sizeof(unsigned int),1,fp);
                unsigned int iTextLength = pButton->m_text.length();
                fwrite(&iTextLength,sizeof(unsigned int),1,fp);
                fwrite(pButton->m_text.c_str(),pButton->m_text.length(),1,fp);
            }
            else if(pItem->IsA(ZUI_SLIDER))
            {
                CZUISlider *pSlider = static_cast<CZUISlider*>(pItem);
                for(int x=0;x<2;x++)
                {
                    fwrite(&pSlider->m_texIDs[x],sizeof(unsigned int),1,fp);
                }
                fwrite(&pSlider->sliderMin,sizeof(int),1,fp);
                fwrite(&pSlider->sliderMax,sizeof(int),1,fp);
                fwrite(&pSlider->sliderPos,sizeof(int),1,fp);
                fwrite(&pSlider->sliderWidth,sizeof(float),1,fp);
                fwrite(&pSlider->sliderHeight,sizeof(float),1,fp);
            }
            else
            {
                fwrite(&pItem->m_texID,sizeof(unsigned int),1,fp);
            }

            fwrite(&pItem->color,sizeof(D3DCOLOR),1,fp);

            pItem->x += m_xoffset;
            pItem->y += m_yoffset;

            fwrite(&pItem->x,sizeof(float),1,fp);
            fwrite(&pItem->y,sizeof(float),1,fp);

            pItem->x -= m_xoffset;
            pItem->y -= m_yoffset;

            fwrite(&pItem->z,sizeof(float),1,fp);
            fwrite(&pItem->width,sizeof(float),1,fp);
            fwrite(&pItem->height,sizeof(float),1,fp);
            int NameLen = pItem->m_name.length();
            fwrite(&NameLen,sizeof(int),1,fp);
            fwrite(pItem->m_name.c_str(),pItem->m_name.length(),1,fp);
        }

        fclose(fp);
    }

    void Load(const char *fileName)
    {
        char buff[255];

        FILE *fp = fopen(fileName,"rb");

        unsigned int totalTextures = 0;
        unsigned int totalItems = 0;
        unsigned int totalFonts = 0;

        fread(&totalTextures,sizeof(unsigned int),1,fp);
        fread(&totalItems,sizeof(unsigned int),1,fp);
        fread(&totalFonts,sizeof(unsigned int),1,fp);

        fread(&m_virtualWidth,sizeof(int),1,fp);
        fread(&m_virtualHeight,sizeof(int),1,fp);

        std::map<int,int> texMapList;
        std::map<int,int> fontMapList;

        for(int x=0;x<totalTextures;x++)
        {
            unsigned int texID;
            int fileNameLen;
            
            bool translucent;
            ZeroMemory(buff,sizeof(buff));

            fread(&texID,sizeof(unsigned int),1,fp);
            fread(&fileNameLen,sizeof(int),1,fp);
            fread(buff,fileNameLen,1,fp);
            fread(&translucent,sizeof(bool),1,fp);

            int newTexID = CreateTexture(buff,translucent);
            texMapList[texID] = newTexID;
        }

        for(x=0;x<totalFonts;x++)
        {
            unsigned int fontID;
            int fontNameLen;
            long color,size,style;

            ZeroMemory(buff,sizeof(buff));
            fread(&fontID,sizeof(unsigned int),1,fp);
            fread(&fontNameLen,sizeof(int),1,fp);
            fread(buff,fontNameLen,1,fp);
            fread(&color,sizeof(long),1,fp);
            fread(&size,sizeof(long),1,fp);
            fread(&style,sizeof(long),1,fp);
            int newFontID = CreateFont(color,buff,size,style);
            fontMapList[fontID] = newFontID;
        }
        
        for(x=0;x<totalItems;x++)
        {
            unsigned int id,texID,itemType;
            int *texIDs = NULL;
            D3DCOLOR color;
            float x,y,z,width,height;
            int NameLen;
            unsigned int fontID;
            unsigned int itemTranslucency;
            std::string caption;
            bool bCallbackEvent;
            int sliderMin,sliderMax,sliderPos;
            float sliderWidth,sliderHeight;


            fread(&id,sizeof(unsigned int),1,fp);
            fread(&itemType,sizeof(unsigned int),1,fp);
            fread(&itemTranslucency,sizeof(unsigned int),1,fp);
            fread(&bCallbackEvent,sizeof(bool),1,fp);
            
            if(itemType == ZUI_CHECKBOX)
            {
                texIDs = new int[6];
                for(int x=0;x<6;x++)
                {
                    fread(&texIDs[x],sizeof(unsigned int),1,fp);
                }
                texID = texIDs[0];
            }
            else if(itemType == ZUI_EDITBOX)
            {
                texIDs = new int[2];
                for(int x=0;x<2;x++)
                {
                    fread(&texIDs[x],sizeof(unsigned int),1,fp);
                }
                texID = texIDs[0];
                fread(&fontID,sizeof(unsigned int),1,fp);
            }
            else if(itemType == ZUI_BUTTON)
            {
                texIDs = new int[2];
                for(int x=0;x<2;x++)
                {
                    fread(&texIDs[x],sizeof(unsigned int),1,fp);
                }
                texID = texIDs[0];
                fread(&fontID,sizeof(unsigned int),1,fp);
                int texLength = 0;
                fread(&texLength,sizeof(unsigned int),1,fp);
                ZeroMemory(buff,255);
                fread(buff,texLength,1,fp);
                caption = buff;
            }
            else if(itemType == ZUI_SLIDER)
            {
                texIDs = new int[2];
                for(int x=0;x<2;x++)
                {
                    fread(&texIDs[x],sizeof(unsigned int),1,fp);
                }
                texID = texIDs[0];
                fread(&sliderMin,sizeof(int),1,fp);
                fread(&sliderMax,sizeof(int),1,fp);
                fread(&sliderPos,sizeof(int),1,fp);

                fread(&sliderWidth,sizeof(float),1,fp);
                fread(&sliderHeight,sizeof(float),1,fp);
            }
            else
            {
                fread(&texID,sizeof(unsigned int),1,fp);
            }
            
            fread(&color,sizeof(D3DCOLOR),1,fp);

            fread(&x,sizeof(float),1,fp);
            fread(&y,sizeof(float),1,fp);

            x -= m_xoffset;
            y -= m_yoffset;

            fread(&z,sizeof(float),1,fp);
            fread(&width,sizeof(float),1,fp);
            fread(&height,sizeof(float),1,fp);
            fread(&NameLen,sizeof(int),1,fp);

            ZeroMemory(buff,sizeof(buff));
            fread(buff,NameLen,1,fp);
            
            int item = CreateItem(x,y,z,width,height,texMapList[texID],itemType,texIDs);
            CZUIItem *pItem = GetItem(item);

            if(itemType == ZUI_EDITBOX)
            {
                CZUIEditBox *pEdit = static_cast<CZUIEditBox*>(pItem);
                pEdit->m_fontID = fontMapList[fontID];
            }
            else if(itemType == ZUI_BUTTON)
            {
                CZUIButton *pButton = static_cast<CZUIButton*>(pItem);
                pButton->m_fontID = fontMapList[fontID];
                pButton->m_text = caption;
            }
            else if(itemType == ZUI_SLIDER)
            {
                CZUISlider *pSlider = static_cast<CZUISlider*>(pItem);
                pSlider->sliderMin = sliderMin;
                pSlider->sliderMax = sliderMax;
                pSlider->sliderPos = sliderPos;
                pSlider->sliderWidth = sliderWidth;
                pSlider->sliderHeight = sliderHeight;
            }

            
            if(NameLen)
                pItem->m_name = buff;
            pItem->m_iTranslucency = itemTranslucency;
            pItem->m_bCallbackEvent = bCallbackEvent;

            pItem->color = D3DCOLOR_ARGB(pItem->m_iTranslucency,255,255,255);
            pItem->Initialize();
            
            delete texIDs;
            texIDs = NULL;
        }

        fclose(fp);
    }

    int GetItemFromPoint(int x,int y)
    {
        float closestZ = 20000.0;
        int   closestItem = -1;

        //find the item that was clicked
        std::vector<CZUIItem *>::iterator iterItem = m_ItemList.begin();
        for(;iterItem != m_ItemList.end(); iterItem++)
        {
            CZUIItem *pItem = (*iterItem);
            if(x>=pItem->x && x <= pItem->x+pItem->width &&
               y>=pItem->y && y <= pItem->y + pItem->height)
            {
                if(pItem->z < closestZ)
                {
                    closestZ = pItem->z;
                    closestItem = pItem->m_ID;
                }
            }
        }
        return closestItem;
    }


    void OnBeforeResize()
    {
        std::vector<CZUIFont*>::iterator iterFont = m_FontList.begin();
        for(;iterFont != m_FontList.end(); iterFont++)
        {
            CZUIFont *pFont = (*iterFont);
            pFont->InvalidateDevice();
        }
    }

    void OnAfterResize()
    {
        std::vector<CZUIFont*>::iterator iterFont =  m_FontList.begin();
        for(;iterFont != m_FontList.end(); iterFont++)
        {
            CZUIFont *pFont = (*iterFont);
            pFont->RestoreDevice();
        }
    }

    void OnMouseDown(int btn,int x,int y)
    {
        if(btn == 1)
        {
            m_bMouseDown = true;
        }
        m_iCaptureItem = GetItemFromPoint(x,y);
        if(m_iCaptureItem != -1)
        {
            CZUIItem *pItem = GetItem(m_iCaptureItem);
            if(pItem)
            {
                pItem->MouseDown(btn,x,y);

                if(pItem->IsA(ZUI_BUTTON))
                    if(eventCallback)
                        eventCallback(pItem);
                
                if(m_iCaptureItem != m_iLastCaptureItem)
                {
                    if(m_iLastCaptureItem != -1)
                    {
                        CZUIItem *pLastItem = GetItem(m_iLastCaptureItem);
                        if(pLastItem->IsA(ZUI_EDITBOX))
                        {
                            CZUIEditBox *pLastEdit = (CZUIEditBox*)pLastItem;
                            pLastEdit->SetSel(-1);
                        }
                    }
                    m_iLastCaptureItem = m_iCaptureItem;
                }
                
                if(pItem->IsA(ZUI_EDITBOX))
                {
                    CZUIEditBox *pEdit = static_cast<CZUIEditBox*>(pItem);
                    if(pEdit->m_text.length() > 0)
                    {
                        CZUIFont *pFont = GetFont(pEdit->m_fontID);
                        if(pFont)
                        {
                            if(pFont->m_pFont)
                            {
                                SIZE sizeXLoc;
                                pFont->m_pFont->GetTextExtent(
                                        (char*)pEdit->m_text.c_str(),&sizeXLoc);

                                pEdit->SetSel(sizeXLoc.cx);
                            }
                        }
                    }
                }
            }
        }
    }

    void OnMouseUp(int btn,int x,int y)
    {
        m_bMouseDown = false;
        if(m_iMouseOverItemID != -1)
        {
            CZUIItem *pItem = GetItem(m_iCaptureItem);
            if(pItem)
            {
                pItem->MouseUp(btn,x,y);
                if(pItem->IsA(ZUI_CHECKBOX))
                    if(eventCallback)
                        eventCallback(pItem);
            }
        }
    }

    void OnMouseMove(int x,int y)
    {
        CZUIItem *pItem = GetItem(GetItemFromPoint(x,y));
        if(pItem)
        {
            if(m_iMouseOverItemID != -1)
                if(m_iMouseOverItemID != pItem->m_ID)
                {
                    GetItem(m_iMouseOverItemID)->MouseOff();
                }

            if(m_bMouseDown)
            {
                pItem->MouseDown(1,x,y);
                if(pItem->IsA(ZUI_SLIDER))
                {
                    if(eventCallback)
                        eventCallback(pItem);
                }
            }
            else
                pItem->MouseMove(x,y);
            m_iMouseOverItemID = pItem->m_ID;
        }
        else
        {
            if(m_iMouseOverItemID != -1)
            {
                pItem = GetItem(m_iMouseOverItemID);
                if(pItem)
                {
                    pItem->MouseOff();
                }
                m_iMouseOverItemID = -1;
            }
        }
    }

    void HandleVKKeyInput(std::string *str,int key, bool keyDown = true)
    {
        static bool shiftDown = false;

        if(key == VK_SHIFT)
        {
            if(keyDown == true)
                shiftDown = true;
            else 
                shiftDown = false;
        }
        
        if(keyDown == false)
            return;

        if(key == VK_BACK)
        {
            if(str->length())
                *str->erase(str->end()-1,str->end());
        }
        else if(key == VK_SPACE)
        {
            *str += " ";
        }
        else if(key >= VK_NUMPAD0 && key <= VK_NUMPAD9)
        {
            *str += key-0x30;
        }
        else if(key >= '0' && key <= '9')
        {
            if(shiftDown == true)
            {
                int i = key - '0';
                switch(i)
                {
                case 0:
                    *str += ")";
                    break;
                case 1:
                    *str += "!";
                    break;
                case 2:
                    *str += "@";
                    break;
                case 3:
                    *str += "#";
                    break;
                case 4:
                    *str += "$";
                    break;
                case 5:
                    *str += "%";
                    break;
                case 6:
                    *str += "^";
                    break;
                case 7:
                    *str += "&";
                    break;
                case 8:
                    *str += "*";
                    break;
                case 9:
                    *str += "(";
                    break;
                }
            }
            else
            {
                *str += key;
            }       
        }
        else if(key >= 'A' && key <= 'Z')
        {
            if(shiftDown == true)
                *str += key;
            else
                *str += key+32;
        }
        else if(key == 0xBA) // ';'
        {
            if(shiftDown)
                *str += ":";
            else
                *str += ";";
        }

        else if(key == 0xBC) // ','
        {
            if(shiftDown)
                *str += "<";
            else
                *str += ",";
        }

        else if(key == 0xBE) // '.'
        {
            if(shiftDown)
                *str += ">";
            else
                *str += ".";
        }
        else if(key == 0xBF) // '/'
        {
            if(shiftDown)
                *str += "?";
            else
                *str += "/";
        }
        else if(key == 0xDE) // '''
        {
            if(shiftDown)
                *str += "\"";
            else
                *str += "'";
        }

        else if(key == 0xDB) // '['
        {
            if(shiftDown)
                *str += "[";
            else
                *str += "{";
        }
        else if(key == 0xDD) // ']'
        {
            if(shiftDown)
                *str += "]";
            else
                *str += "}";
        }
        else if(key == 0xC0) // '`'
        {
            if(shiftDown)
                *str += "~";
            else
                *str += "`";
        }

        else if(key == 0xBD) // '-'
        {
            if(shiftDown)
                *str += "_";
            else
                *str += "-";
        }

        else if(key == 0xBB) // '='
        {
            if(shiftDown)
                *str += "+";
            else
                *str += "=";
        }
    }

    void VKKeyHandle(int key, bool keyDown)
    {
        if(m_iCaptureItem != -1)
        {
            CZUIItem *pItem = GetItem(m_iCaptureItem);
            if(pItem)
            {
                if(pItem->IsA(ZUI_EDITBOX))
                {
                    CZUIEditBox *pEdit = (CZUIEditBox*)pItem;
                    
                    HandleVKKeyInput(&pEdit->m_text,key,keyDown);

                    if(pEdit->m_text.length() > 0)
                    {
                        CZUIFont *pFont = GetFont(pEdit->m_fontID);
                        if(pFont)
                        {
                            if(pFont->m_pFont)
                            {
                                SIZE sizeXLoc;
                                pFont->m_pFont->GetTextExtent(
                                        (char*)pEdit->m_text.c_str(),&sizeXLoc);

                                pEdit->SetSel(sizeXLoc.cx);
                            }
                        }
                    }
                    if(pEdit->m_name.length() > 0)
                    {
                        if(pEdit->m_bCallbackEvent)
                        {
                            if(eventCallback)
                                eventCallback(pItem);
                        }
                    }
                }
            }
        }
    }
};
